/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NIADDITIONALGEOMETRYDATA_H_
#define _NIADDITIONALGEOMETRYDATA_H_

//--BEGIN FILE HEAD CUSTOM CODE--//

//--END CUSTOM CODE--//

#include "AbstractAdditionalGeometryData.h"

// Include structures
#include "../gen/AdditionalDataInfo.h"
#include "../gen/AdditionalDataBlock.h"
namespace Niflib {

class NiAdditionalGeometryData;
typedef Ref<NiAdditionalGeometryData> NiAdditionalGeometryDataRef;

/*!  */
class NiAdditionalGeometryData : public AbstractAdditionalGeometryData {
public:
	/*! Constructor */
	NIFLIB_API NiAdditionalGeometryData();

	/*! Destructor */
	NIFLIB_API virtual ~NiAdditionalGeometryData();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// Number of vertices
	// \return The current value.
	unsigned short GetNumVertices() const;

	// Number of vertices
	// \param[in] value The new value.
	void SetNumVertices( unsigned short value );

	// Number of additional data blocks
	// \return The current value.
	vector<AdditionalDataInfo > GetBlockInfos() const;

	// Number of additional data blocks
	// \param[in] value The new value.
	void SetBlockInfos( const vector<AdditionalDataInfo >& value );

	// Number of additional data blocks
	// \return The current value.
	vector<AdditionalDataBlock > GetBlocks() const;

	// Number of additional data blocks
	// \param[in] value The new value.
	void SetBlocks( const vector<AdditionalDataBlock >& value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//

	//--END CUSTOM CODE--//
protected:
	/*! Number of vertices */
	unsigned short numVertices;
	/*! Information about additional data blocks */
	mutable unsigned int numBlockInfos;
	/*! Number of additional data blocks */
	vector<AdditionalDataInfo > blockInfos;
	/*! Number of additional data blocks */
	mutable int numBlocks;
	/*! Number of additional data blocks */
	vector<AdditionalDataBlock > blocks;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//

//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
