/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NIPSYSBOMBMODIFIER_H_
#define _NIPSYSBOMBMODIFIER_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiPSysModifier.h"
namespace Niflib {

// Forward define of referenced NIF objects
class NiNode;
class NiPSysBombModifier;
typedef Ref<NiPSysBombModifier> NiPSysBombModifierRef;

/*!
 * Particle modifier that uses a NiNode to use as a "Bomb Object" to alter the path
 * of particles.
 */
class NiPSysBombModifier : public NiPSysModifier {
public:
	/*! Constructor */
	NIFLIB_API NiPSysBombModifier();

	/*! Destructor */
	NIFLIB_API virtual ~NiPSysBombModifier();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// Link to a NiNode for bomb to function.
	// \return The current value.
	NiNode * GetBombObject() const;

	// Link to a NiNode for bomb to function.
	// \param[in] value The new value.
	void SetBombObject( NiNode * value );

	// Orientation of bomb object.
	// \return The current value.
	Vector3 GetBombAxis() const;

	// Orientation of bomb object.
	// \param[in] value The new value.
	void SetBombAxis( const Vector3 & value );

	// Falloff rate of the bomb object.
	// \return The current value.
	float GetDecay() const;

	// Falloff rate of the bomb object.
	// \param[in] value The new value.
	void SetDecay( float value );

	// DeltaV /  Strength?
	// \return The current value.
	float GetDeltaV() const;

	// DeltaV /  Strength?
	// \param[in] value The new value.
	void SetDeltaV( float value );

	// Decay type
	// \return The current value.
	DecayType GetDecayType() const;

	// Decay type
	// \param[in] value The new value.
	void SetDecayType( const DecayType & value );

	// Shape/symmetry of the bomb object.
	// \return The current value.
	SymmetryType GetSymmetryType() const;

	// Shape/symmetry of the bomb object.
	// \param[in] value The new value.
	void SetSymmetryType( const SymmetryType & value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//
	//--END CUSTOM CODE--//
protected:
	/*! Link to a NiNode for bomb to function. */
	NiNode * bombObject;
	/*! Orientation of bomb object. */
	Vector3 bombAxis;
	/*! Falloff rate of the bomb object. */
	float decay;
	/*! DeltaV /  Strength? */
	float deltaV;
	/*! Decay type */
	DecayType decayType;
	/*! Shape/symmetry of the bomb object. */
	SymmetryType symmetryType;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
