/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NIPALETTE_H_
#define _NIPALETTE_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiObject.h"

// Include structures
#include "../gen/ByteColor4.h"
namespace Niflib {

class NiPalette;
typedef Ref<NiPalette> NiPaletteRef;

/*! A color palette. */
class NiPalette : public NiObject {
public:
	/*! Constructor */
	NIFLIB_API NiPalette();

	/*! Destructor */
	NIFLIB_API virtual ~NiPalette();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// The number of palette entries.  Always = 256.
	// \return The current value.
	unsigned int GetNumEntries() const;

	// The number of palette entries.  Always = 256.
	// \param[in] value The new value.
	void SetNumEntries( unsigned int value );

	// The color palette.
	// \return The current value.
	array<256,ByteColor4 >  GetPalette() const;

	// The color palette.
	// \param[in] value The new value.
	void SetPalette( const array<256,ByteColor4 >&  value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//

	/*!
	 * Retrieves the palette data from this palette object.
	 * \return A vector containing the the colors stored in the palette.
	 * \sa NiPalette::SetPalette
	 */
	NIFLIB_API vector<Color4> GetPalette() const;

	/*!
	 * Sets the palette data for this palette object.
	 * \param new_pal A vector containing the the new colors to be stored in the palette.
	 * \sa NiPalette::GetPalette
	 */
	NIFLIB_API void SetPalette( const vector<Color4> & new_pal );

	//--END CUSTOM CODE--//
protected:
	/*! Unknown, Usually = 0. */
	byte unknownByte;
	/*! The number of palette entries.  Always = 256. */
	unsigned int numEntries;
	/*! The color palette. */
	array<256,ByteColor4 > palette;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
