/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NIPARTICLEGROWFADE_H_
#define _NIPARTICLEGROWFADE_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiParticleModifier.h"
namespace Niflib {

class NiParticleGrowFade;
typedef Ref<NiParticleGrowFade> NiParticleGrowFadeRef;

/*!
 * This particle system modifier controls the particle size. If it is present the
 * particles start with size 0.0 . Then they grow to their original size and stay
 * there until they fade to zero size again at the end of their lifetime cycle.
 */
class NiParticleGrowFade : public NiParticleModifier {
public:
	/*! Constructor */
	NIFLIB_API NiParticleGrowFade();

	/*! Destructor */
	NIFLIB_API virtual ~NiParticleGrowFade();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// The time from the beginning of the particle lifetime during which the particle
	// grows.
	// \return The current value.
	float GetGrow() const;

	// The time from the beginning of the particle lifetime during which the particle
	// grows.
	// \param[in] value The new value.
	void SetGrow( float value );

	// The time from the end of the particle lifetime during which the particle fades.
	// \return The current value.
	float GetFade() const;

	// The time from the end of the particle lifetime during which the particle fades.
	// \param[in] value The new value.
	void SetFade( float value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//
	//--END CUSTOM CODE--//
protected:
	/*!
	 * The time from the beginning of the particle lifetime during which the particle
	 * grows.
	 */
	float grow;
	/*! The time from the end of the particle lifetime during which the particle fades. */
	float fade;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
