/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NIRAWIMAGEDATA_H_
#define _NIRAWIMAGEDATA_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiObject.h"

// Include structures
#include "../gen/ByteColor3.h"
#include "../gen/ByteColor4.h"
namespace Niflib {

class NiRawImageData;
typedef Ref<NiRawImageData> NiRawImageDataRef;

/*! Raw image data. */
class NiRawImageData : public NiObject {
public:
	/*! Constructor */
	NIFLIB_API NiRawImageData();

	/*! Destructor */
	NIFLIB_API virtual ~NiRawImageData();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// The format of the raw image data.
	// \return The current value.
	ImageType GetImageType() const;

	// The format of the raw image data.
	// \param[in] value The new value.
	void SetImageType( const ImageType & value );

	// Image pixel data.
	// \return The current value.
	vector<vector<ByteColor3 > > GetRgbImageData() const;

	// Image pixel data.
	// \param[in] value The new value.
	void SetRgbImageData( const vector<ByteColor3 >& value );

	// Image pixel data.
	// \return The current value.
	vector<vector<ByteColor4 > > GetRgbaImageData() const;

	// Image pixel data.
	// \param[in] value The new value.
	void SetRgbaImageData( const vector<ByteColor4 >& value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//
	//--END CUSTOM CODE--//
protected:
	/*! Image width */
	mutable unsigned int width;
	/*! Image height */
	mutable unsigned int height;
	/*! The format of the raw image data. */
	ImageType imageType;
	/*! Image pixel data. */
	vector< vector<ByteColor3 > > rgbImageData;
	/*! Image pixel data. */
	vector< vector<ByteColor4 > > rgbaImageData;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
