/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NISCREENLODDATA_H_
#define _NISCREENLODDATA_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiLODData.h"
namespace Niflib {

class NiScreenLODData;
typedef Ref<NiScreenLODData> NiScreenLODDataRef;

/*! Describes levels of detail based on size of object on screen? */
class NiScreenLODData : public NiLODData {
public:
	/*! Constructor */
	NIFLIB_API NiScreenLODData();

	/*! Destructor */
	NIFLIB_API virtual ~NiScreenLODData();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// The center of the bounding sphere?
	// \return The current value.
	Vector3 GetBoundCenter() const;

	// The center of the bounding sphere?
	// \param[in] value The new value.
	void SetBoundCenter( const Vector3 & value );

	// The radius of the bounding sphere?
	// \return The current value.
	float GetBoundRadius() const;

	// The radius of the bounding sphere?
	// \param[in] value The new value.
	void SetBoundRadius( float value );

	// The center of the bounding sphere in world space?
	// \return The current value.
	Vector3 GetWorldCenter() const;

	// The center of the bounding sphere in world space?
	// \param[in] value The new value.
	void SetWorldCenter( const Vector3 & value );

	// The radius of the bounding sphere in world space?
	// \return The current value.
	float GetWorldRadius() const;

	// The radius of the bounding sphere in world space?
	// \param[in] value The new value.
	void SetWorldRadius( float value );

	// The LOD levels based on proportion of screen size?
	// \return The current value.
	vector<float > GetProportionLevels() const;

	// The LOD levels based on proportion of screen size?
	// \param[in] value The new value.
	void SetProportionLevels( const vector<float >& value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//

	/*!
	 * Get the center of the bounding sphere?
	 * \return The center of the bounding sphere?
	 */
	NIFLIB_API Vector3 GetBoundCenter() const;

	/*!
	 * Set the center of the bounding sphere?
	 * \param[in] bound_center The new center of the bounding sphere?
	 */
	NIFLIB_API void SetBoundCenter( const Vector3 & bound_center );

	/*!
	 * Get the radius of the bounding sphere?
	 * \return The radius of the bounding sphere?
	 */
	NIFLIB_API float GetBoundRadius() const;

	/*!
	 * Set the radius of the bounding sphere?
	 * \param[in] value The new radius of the bounding sphere?
	 */
	NIFLIB_API void SetBoundRadius( float value );

	/*!
	 * Get the center of the bounding sphere in world space?
	 * \return The center of the bounding sphere in world space?
	 */
	NIFLIB_API Vector3 GetWorldCenter() const;

	/*!
	 * Set the center of the bounding sphere in world space?
	 * \param[in] value The new center of the bounding sphere in world space?
	 */
	NIFLIB_API void SetWorldCenter( const Vector3 & value );

	/*!
	 * Get the radius of the bounding sphere in world space?
	 * \return The radius of the bounding sphere in world space?
	 */
	NIFLIB_API float GetWorldRadius() const;

	/*!
	 * Set the radius of the bounding sphere in world space?
	 * \param[in] value The new radius of the bounding sphere in world space?
	 */
	NIFLIB_API void SetWorldRadius( float value );

	/*!
	 * Get the LOD levels based on proportion of screen size?
	 * \return The LOD levels based on proportion of screen size?
	 */
	NIFLIB_API vector<float > GetProportionLevels() const;

	/*!
	 * Set the LOD levels based on proportion of screen size?
	 * \param[in] value The LOD levels based on proportion of screen size?
	 */
	NIFLIB_API void SetProportionLevels( const vector<float> & value );

	//--END CUSTOM CODE--//
protected:
	/*! The center of the bounding sphere? */
	Vector3 boundCenter;
	/*! The radius of the bounding sphere? */
	float boundRadius;
	/*! The center of the bounding sphere in world space? */
	Vector3 worldCenter;
	/*! The radius of the bounding sphere in world space? */
	float worldRadius;
	/*! The number of screen size based LOD levels. */
	mutable unsigned int proportionCount;
	/*! The LOD levels based on proportion of screen size? */
	vector<float > proportionLevels;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
