/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NISTRINGEXTRADATA_H_
#define _NISTRINGEXTRADATA_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiExtraData.h"
namespace Niflib {

class NiStringExtraData;
typedef Ref<NiStringExtraData> NiStringExtraDataRef;

/*!
 * Apparently commands for an optimizer instructing it to keep things it would
 * normally discard.
 *         Also refers to NiNode objects (through their name) in animation .kf
 * files.
 */
class NiStringExtraData : public NiExtraData {
public:
	/*! Constructor */
	NIFLIB_API NiStringExtraData();

	/*! Destructor */
	NIFLIB_API virtual ~NiStringExtraData();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// The number of bytes left in the record.  Equals the length of the following
	// string + 4.
	// \return The current value.
	unsigned int GetBytesRemaining() const;

	// The number of bytes left in the record.  Equals the length of the following
	// string + 4.
	// \param[in] value The new value.
	void SetBytesRemaining( unsigned int value );

	// The string.
	// \return The current value.
	IndexString GetStringData() const;

	// The string.
	// \param[in] value The new value.
	void SetStringData( const IndexString & value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//

	/*!
	 * Retrieves the string value stored in this object.
	 * \return The string value stored in this object.
	 */
	NIFLIB_API string GetData() const;

	/*!
	 * Sets the string value stored in this object.
	 * \param[in] n The new string value to store in this object.
	 */
	NIFLIB_API void SetData( const string & n );

	//--END CUSTOM CODE--//
protected:
	/*!
	 * The number of bytes left in the record.  Equals the length of the following
	 * string + 4.
	 */
	unsigned int bytesRemaining;
	/*! The string. */
	IndexString stringData;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
