/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _NITEXTUREPROPERTY_H_
#define _NITEXTUREPROPERTY_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "NiProperty.h"

// Include structures
#include "../Ref.h"
namespace Niflib {

// Forward define of referenced NIF objects
class NiImage;
class NiTextureProperty;
typedef Ref<NiTextureProperty> NiTexturePropertyRef;

/*!  */
class NiTextureProperty : public NiProperty {
public:
	/*! Constructor */
	NIFLIB_API NiTextureProperty();

	/*! Destructor */
	NIFLIB_API virtual ~NiTextureProperty();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// Property flags.
	// \return The current value.
	unsigned short GetFlags() const;

	// Property flags.
	// \param[in] value The new value.
	void SetFlags( unsigned short value );

	// Link to the texture image.
	// \return The current value.
	Ref<NiImage > GetImage() const;

	// Link to the texture image.
	// \param[in] value The new value.
	void SetImage( Ref<NiImage > value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//

	/*!
	 * Retrieves the image object used by this texture property, if any.
	 * \return The image used by this property, or NULL if there is none.
	 */
	NIFLIB_API Ref<NiImage> GetImage() const;

	/*!
	 * Sets the image object used by this texture property, if any.
	 * \return The new image to be used by this property, or NULL to clear the existing one.
	 */
	NIFLIB_API void SetImage( NiImage * n );

	//--END CUSTOM CODE--//
protected:
	/*! Property flags. */
	array<2,unsigned int > unknownInts1;
	/*! Property flags. */
	unsigned short flags;
	/*! Link to the texture image. */
	Ref<NiImage > image;
	/*! Unknown.  0? */
	array<2,unsigned int > unknownInts2;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
