/* Copyright (c) 2006, NIF File Format Library and Tools
All rights reserved.  Please see niflib.h for license. */

//-----------------------------------NOTICE----------------------------------//
// Some of this file is automatically filled in by a Python script.  Only    //
// add custom code in the designated areas or it will be overwritten during  //
// the next update.                                                          //
//-----------------------------------NOTICE----------------------------------//

#ifndef _BHKPRISMATICCONSTRAINT_H_
#define _BHKPRISMATICCONSTRAINT_H_

//--BEGIN FILE HEAD CUSTOM CODE--//
//--END CUSTOM CODE--//

#include "bhkConstraint.h"
namespace Niflib {

class bhkPrismaticConstraint;
typedef Ref<bhkPrismaticConstraint> bhkPrismaticConstraintRef;

/*! A prismatic constraint. */
class bhkPrismaticConstraint : public bhkConstraint {
public:
	/*! Constructor */
	NIFLIB_API bhkPrismaticConstraint();

	/*! Destructor */
	NIFLIB_API virtual ~bhkPrismaticConstraint();

	/*!
	 * A constant value which uniquly identifies objects of this type.
	 */
	NIFLIB_API static const Type TYPE;

	/*!
	 * A factory function used during file reading to create an instance of this type of object.
	 * \return A pointer to a newly allocated instance of this type of object.
	 */
	NIFLIB_API static NiObject * Create();

	/*!
	 * Summarizes the information contained in this object in English.
	 * \param[in] verbose Determines whether or not detailed information about large areas of data will be printed out.
	 * \return A string containing a summary of the information within the object in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 */
	NIFLIB_API virtual string asString( bool verbose = false ) const;

	/*!
	 * Used to determine the type of a particular instance of this object.
	 * \return The type constant for the actual type of the object.
	 */
	NIFLIB_API virtual const Type & GetType() const;

	/***Begin Example Naive Implementation****

	// Pivot A.
	// \return The current value.
	Vector4 GetPivotA() const;

	// Pivot A.
	// \param[in] value The new value.
	void SetPivotA( const Vector4 & value );

	// 4x4 rotation matrix, rotates the child entity.
	// \return The current value.
	array<4,Vector4 >  GetRotationMatrixA() const;

	// 4x4 rotation matrix, rotates the child entity.
	// \param[in] value The new value.
	void SetRotationMatrixA( const array<4,Vector4 >&  value );

	// Pivot B.
	// \return The current value.
	Vector4 GetPivotB() const;

	// Pivot B.
	// \param[in] value The new value.
	void SetPivotB( const Vector4 & value );

	// Describes the axis the object is able to travel along. Unit vector.
	// \return The current value.
	Vector4 GetSlidingB() const;

	// Describes the axis the object is able to travel along. Unit vector.
	// \param[in] value The new value.
	void SetSlidingB( const Vector4 & value );

	// Plane normal. Describes the plane the object is able to move on.
	// \return The current value.
	Vector4 GetPlaneB() const;

	// Plane normal. Describes the plane the object is able to move on.
	// \param[in] value The new value.
	void SetPlaneB( const Vector4 & value );

	// Describes the axis the object is able to travel along. Unit vector.
	// \return The current value.
	Vector4 GetSlidingA() const;

	// Describes the axis the object is able to travel along. Unit vector.
	// \param[in] value The new value.
	void SetSlidingA( const Vector4 & value );

	// Rotation axis.
	// \return The current value.
	Vector4 GetRotationA() const;

	// Rotation axis.
	// \param[in] value The new value.
	void SetRotationA( const Vector4 & value );

	// Plane normal. Describes the plane the object is able to move on.
	// \return The current value.
	Vector4 GetPlaneA() const;

	// Plane normal. Describes the plane the object is able to move on.
	// \param[in] value The new value.
	void SetPlaneA( const Vector4 & value );

	// Pivot.
	// \return The current value.
	Vector4 GetPivotA() const;

	// Pivot.
	// \param[in] value The new value.
	void SetPivotA( const Vector4 & value );

	// Describes the axis the object is able to travel along in B coordinates. Unit
	// vector.
	// \return The current value.
	Vector4 GetSlidingB() const;

	// Describes the axis the object is able to travel along in B coordinates. Unit
	// vector.
	// \param[in] value The new value.
	void SetSlidingB( const Vector4 & value );

	// Rotation axis.
	// \return The current value.
	Vector4 GetRotationB() const;

	// Rotation axis.
	// \param[in] value The new value.
	void SetRotationB( const Vector4 & value );

	// Plane normal. Describes the plane the object is able to move on in B
	// coordinates.
	// \return The current value.
	Vector4 GetPlaneB() const;

	// Plane normal. Describes the plane the object is able to move on in B
	// coordinates.
	// \param[in] value The new value.
	void SetPlaneB( const Vector4 & value );

	// Pivot in B coordinates.
	// \return The current value.
	Vector4 GetPivotB() const;

	// Pivot in B coordinates.
	// \param[in] value The new value.
	void SetPivotB( const Vector4 & value );

	// Describe the min distance the object is able to travel.
	// \return The current value.
	float GetMinDistance() const;

	// Describe the min distance the object is able to travel.
	// \param[in] value The new value.
	void SetMinDistance( float value );

	// Describe the max distance the object is able to travel.
	// \return The current value.
	float GetMaxDistance() const;

	// Describe the max distance the object is able to travel.
	// \param[in] value The new value.
	void SetMaxDistance( float value );

	// Friction.
	// \return The current value.
	float GetFriction() const;

	// Friction.
	// \param[in] value The new value.
	void SetFriction( float value );

	****End Example Naive Implementation***/

	//--BEGIN MISC CUSTOM CODE--//
	//--END CUSTOM CODE--//
protected:
	/*! Pivot A. */
	Vector4 pivotA;
	/*! 4x4 rotation matrix, rotates the child entity. */
	array<4,Vector4 > rotationMatrixA;
	/*! Pivot B. */
	Vector4 pivotB;
	/*! Describes the axis the object is able to travel along. Unit vector. */
	Vector4 slidingB;
	/*! Plane normal. Describes the plane the object is able to move on. */
	Vector4 planeB;
	/*! Describes the axis the object is able to travel along. Unit vector. */
	Vector4 slidingA;
	/*! Rotation axis. */
	Vector4 rotationA;
	/*! Plane normal. Describes the plane the object is able to move on. */
	Vector4 planeA;
	/*! Rotation axis. */
	Vector4 rotationB;
	/*! Describe the min distance the object is able to travel. */
	float minDistance;
	/*! Describe the max distance the object is able to travel. */
	float maxDistance;
	/*! Friction. */
	float friction;
	/*! Unknown. Do not set this to anything over 0 as it will crash the game. */
	byte unknownByte1;
public:
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Read( istream& in, list<unsigned int> & link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void Write( ostream& out, const map<NiObjectRef,unsigned int> & link_map, list<NiObject *> & missing_link_stack, const NifInfo & info ) const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual void FixLinks( const map<unsigned int,NiObjectRef> & objects, list<unsigned int> & link_stack, list<NiObjectRef> & missing_link_stack, const NifInfo & info );
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObjectRef> GetRefs() const;
	/*! NIFLIB_HIDDEN function.  For internal use only. */
	NIFLIB_HIDDEN virtual list<NiObject *> GetPtrs() const;
};

//--BEGIN FILE FOOT CUSTOM CODE--//
//--END CUSTOM CODE--//

} //End Niflib namespace
#endif
